/*
 * This file is part of Firestorm NIDS
 * Copyright (c) 2004 Gianni Tedesco
 * Released under the terms of the GNU GPL version 2
 *
 * Linux epoll based eventloop
 */

#if HAVE_SYS_EPOLL_H
#include <sys/epoll.h>
#endif

static int epoll_init(struct iothread *t)
{
	t->priv.epoll = epoll_create(0);
	if ( t->priv.epoll < 0 )
		return 0;
	return 1;
}

static void epoll_fini(struct iothread *t)
{
	fd_close(t->priv.epoll);
}

static void epoll_active(struct iothread *t, struct nbio *n)
{
	if ( n->ev_priv.poll == 0 )
		return;

	n->ev_priv.poll = 0;
	epoll_ctl(t->priv.epoll, EPOLL_CTL_DEL, n->fd, NULL);
}

static void epoll_pump(struct iothread *t)
{
	struct epoll_event ev[8];
	struct nbio *n;
	int nfd, i;

	list_for_each_entry(n, &t->active, list)
		epoll_active(t, n);

	nfd = epoll_wait(t->priv.epoll, ev, sizeof(ev)/sizeof(*ev), -1);
	if ( nfd < 0 )
		return;

	for(i=0; i < nfd; i++) {
		n = ev[i].data.ptr;
		n->flags = 0;
		if ( ev[i].events & EPOLLIN )
			n->flags |= NBIO_READ;
		if ( ev[i].events & EPOLLOUT )
			n->flags |= NBIO_WRITE;
		if ( ev[i].events & (EPOLLERR|EPOLLHUP) )
			n->flags |= NBIO_ERROR;
		
		list_move_tail(&n->list, &t->active);
	}
}

static void epoll_inactive(struct iothread *t, struct nbio *n)
{
	struct epoll_event ev;

	if ( n->ev_priv.poll == 1 )
		return;

	n->ev_priv.poll = 1;

	ev.events = EPOLLERR|EPOLLHUP|EPOLLET;
	ev.data.ptr = n;

	if ( n->mask & NBIO_READ )
		ev.events |= EPOLLIN;
	if ( n->mask & NBIO_WRITE )
		ev.events |= EPOLLOUT;

	epoll_ctl(t->priv.epoll, EPOLL_CTL_ADD, n->fd, &ev);
}
