/*
 * This file is part of Firestorm NIDS
 * Copyright (c) 2002,2003,2004 Gianni Tedesco <gianni@scaramanga.co.uk>
 * This program is released under the terms of the GNU GPL version 2
*/
#include "tcpip.h"
#include <errno.h>

/* Generator */
struct generator icmp_gen=init_generator("sig.icmp", NULL);

static struct sub_table icmp_subs[]={
	{"src", "ip.src"},
	{"dst", "ip.dst"},
	{"sport", "icmp.type"},
	{"dport", "icmp.code"},
	{"itype", "icmp.type"},
	{"icode", "icmp.code"},
	{"icmp_id", "icmp.id"},
	{"icmp_seq", "icmp.seq"},
	{"tos", "ip.tos"},
	{"id", "ip.id"},
	{"fragoffset", "ip.fragofs"},
	{"fragbits", "ip.fragbits"},
	{"ttl", "ip.ttl"},
	{"ip_proto", "ip.proto"},
	{"sameip", "ip.sameip"},
	{"ipopts", "ip.opts"},
	{NULL, NULL},
};
struct proto icmp_p={
	.name		="icmp",
	.decode		= icmp_decode,
	.print		= icmp_dprint,
	.sig_add	= icmp_commit,
	.sig_match	= icmp_match,
	.sig_subs	= icmp_subs,
	.fields		= icmp_fields,
	.tag		= icmp_tag,
};

struct proto_req icmp_r[]={
	proto_request("ip", 1),
	null_request()
};

void icmp_tag(struct packet *p, int l, struct alert_tag *t)
{
	mesg(M_DEBUG, "ICMP packet tagged");
}

int icmp_dprint(struct packet *p, int l, char *buf, size_t buflen)
{
	struct layer *a = &p->layer[l];
	return snprintf(buf, buflen,
		"type=%u code=%u",
		a->h.icmp->type,
		a->h.icmp->code);
}

void icmp_decode(struct packet *p)
{
	int my_layer=p->llen;
	struct layer *l=&p->layer[p->llen];
	struct pkt_icmphdr *icmph=l->h.icmp;

	if ( (p->layer[p->llen+1].h.raw=
		l->h.raw+sizeof(struct pkt_icmphdr)) > p->end )
		goto finish;

	p->llen++;

	if ( p->llen >= PKT_LAYERS ) {
		dispatch(p);
		return;
	}

	/* ICMP error codes contain chunks of
	 * the IP packet that caused the error
	 */
	if ( icmph->type==ICMP_DEST_UNREACH ||
	     icmph->type==ICMP_SOURCE_QUENCH ||
	     icmph->type==ICMP_TIME_EXCEEDED ||
	     icmph->type==ICMP_PARAMETERPROB) {
		p->layer[p->llen].flags=0;
		p->layer[p->llen].session=NULL;
		p->layer[p->llen].proto=&ipv4_p;
		ipv4_p.decode(p);
		if ( tcp_stateful )
			icmperr_process(p, my_layer);
		return;
	}

finish:
	if ( p->layer[p->llen].h.raw<p->end )
		p->layer[p->llen++].proto=NULL;
	dispatch(p);
}

/* ===================================================
 * PACKET MATCHING STUFF BEYOND THIS POINT
 * ===================================================
 */

static struct detect_root icmp_root;

void icmp_match(struct packet *p, int l)
{
	detect(&icmp_root, p, --l);
}

int icmp_commit(struct rule *r)
{
	struct alert *a;
	struct sig_node *x;

	x = detect_new_sig(r, NULL);
	if ( x == NULL )
		return 0;

	a = calloc(1, sizeof(*a));
	if ( a == NULL ) {
		detect_free_sig(x, r->num_criteria);
		return 0;
	}

	memcpy(a, &r->alert, sizeof(*a));
	a->gen = &icmp_gen;

	return detect_add_sig(&icmp_root, x, r->num_criteria, a);
}
