/*
 * This file is part of Firestorm NIDS
 * Copyright (c) 2002,2003,2004 Gianni Tedesco <gianni@scaramanga.co.uk>
 * This program is released under the terms of the GNU GPL version 2
*/
#ifndef __PLUGIN_TCPIP__
#define __PLUGIN_TCPIP__

#include <firestorm.h>

#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include <dotscara/fobuf.h>
#include <alert.h>
#include <signature.h>
#include <plugin.h>
#include <args.h>
#include <decode.h>
#include <preproc.h>
#include <args.h>
#include <detect.h>
#include <idx.h>
#include <i_idx.h>
#include <q_idx.h>
#include <q_filter.h>
#include <matcher.h>
#include <field.h>

extern unsigned short ipfmask;
extern unsigned int use_ipfrag;
extern unsigned int tcp_stateful;

void tcpstream_tag_session(struct packet *p, int l, struct alert_tag *t);

void ipfrag_csum(struct pkt_iphdr *iph);

int ipfrag_init(const char *, preproc_fn, void *);
int tcpstream_init(const char *, preproc_fn, void *);

int ipfrag_process(struct packet *, int);
void tcpstream_process(struct packet *, int);
void icmperr_process(struct packet *, int);

void ipfrag_free(void);
void tcpstream_free(void);

void ipq_kill(struct ipq *) __nonull(1);
void tcp_free(struct tcp_session *) __nonull(1);

/* Protocols */
extern struct proto ipv4_p;
extern struct proto icmp_p;
extern struct proto tcp_p;
extern struct proto udp_p;

/* Protocol requests */
extern struct proto_req ipv4_r[];
extern struct proto_req icmp_r[];
extern struct proto_req tcp_r[];
extern struct proto_req udp_r[];

/* Decode functions */
void ipv4_decode(struct packet *);
void icmp_decode(struct packet *);
void tcp_decode(struct packet *);
void udp_decode(struct packet *);

/* Print functions */
int ipv4_dprint(struct packet *, int, char *, size_t);
int icmp_dprint(struct packet *, int, char *, size_t);
int tcp_dprint(struct packet *, int, char *, size_t);
int udp_dprint(struct packet *, int, char *, size_t);

/* Generators */
extern struct generator ip_gen;
extern struct generator dip_gen;
extern struct generator tcp_gen;
extern struct generator udp_gen;
extern struct generator icmp_gen;
extern struct generator ipfrag_gen;
extern struct generator tcpstream_gen;

/* Token buckets for alert rate-limiting */
extern struct tokenbucket ip_tb;

/* Fields for indexing */
extern struct field ip_fields[];
extern struct field tcp_fields[];
extern struct field udp_fields[];
extern struct field icmp_fields[];

/* Match functions */
void ip_match(struct packet *, int);
void tcp_match(struct packet *, int);
void udp_match(struct packet *, int);
void icmp_match(struct packet *, int);

/* Rule commital */
int ip_commit(struct rule *);
int tcp_commit(struct rule *);
int udp_commit(struct rule *);
int icmp_commit(struct rule *);

void ip_tag(struct packet *p, int l, struct alert_tag *t);
void udp_tag(struct packet *p, int l, struct alert_tag *t);
void tcp_tag(struct packet *p, int l, struct alert_tag *t);
void icmp_tag(struct packet *p, int l, struct alert_tag *t);

extern struct matcher ip_src_matcher;
extern struct matcher ip_dst_matcher;
extern struct matcher ip_opts_matcher;
extern struct matcher ip_fragbits_matcher;
extern struct matcher tcp_flow_matcher;
extern struct matcher tcp_stateless_matcher;
extern struct matcher tcp_flags_matcher;

extern struct proto_field ip_src_field;
extern struct proto_field ip_dst_field;
extern struct proto_field tcp_flags_field;

#endif /* __PLUGIN_TCPIP__ */
