/*
 * This file is part of Firestorm NIDS
 * Copyright (c) 2002,2003,2004 Gianni Tedesco
 * This program is released under the terms of the GNU GPL version 2
*/

#include <firestorm.h>
#include <errno.h>
#include <alert.h>
#include <signature.h>
#include <decode.h>
#include <matcher.h>
#include <plugin.h>

struct dnsr_detect {
	struct detect d;
	struct {
		struct alert *alert;
		struct detect *result;
	}recurse[2];
};

struct dns_hdr {
	u_int16_t	id;
	u_int16_t	flags;
	u_int16_t	questions;
	u_int16_t	answers;
	u_int16_t	auth;
	u_int16_t	additional;
};

static const u_int16_t dns_qflag=__constant_htons(1<<15);
static const u_int16_t dns_rflag=__constant_htons(1<<8);

static void dnsr_detect(struct detect *d, struct packet *pkt, int l)
{
	struct dnsr_detect *x = (struct dnsr_detect *)d;
	struct dns_hdr *dnshdr;
	int i, tagl;

	tagl = l;
	l += 2;

	if ( l >= pkt->llen )
		goto skip;
	if ( (pkt->layer[l].h.raw + sizeof(*dnshdr)) >= pkt->end )
		goto skip;

	dnshdr = pkt->layer[l].h.raw;

	if ( (dnshdr->flags & dns_qflag) )
		goto skip;

	i = ((dnshdr->flags & dns_rflag) != 0);

	if ( x->recurse[i].alert )
		alert_tag(pkt, x->recurse[i].alert, tagl);
	if ( x->recurse[i].result )
		x->recurse[i].result->match(x->recurse[i].result, pkt, tagl);

skip:
	if ( x->any )
		x->any->match(x->any, pkt, tagl);
}

static struct detect **dnsr_insert(struct detect *d, struct sig_node *n)
{
	struct dnsr_detect *x = (struct dnsr_detect *)d;
	int i = !n->negate;

	x->recurse[i].alert = alert_best(x->recurse[i].alert, n->alert);
	return &x->recurse[i].result;
}

static struct detect **dnsi_insert(struct detect *d, struct sig_node *n)
{
	struct dnsr_detect *x = (struct dnsr_detect *)d;
	int i = !!n->negate;

	x->recurse[i].alert = alert_best(x->recurse[i].alert, n->alert);
	return &x->recurse[i].result;
}

static struct detect *dnsr_new(void)
{
	struct dnsr_detect *d;

	 d = calloc(1, sizeof(*d));
	 if ( d == NULL )
		 return NULL;

	 d->d.match = dnsr_detect;

	 return (struct detect *)d;
};

static int dnsr_validate(char *args, void **priv, struct modifier *m)
{
	if ( args )
		return 0;
	*priv = NULL;
	return 1;
}
static int dnsi_validate(char *args, void **priv, struct modifier *m)
{
	if ( args )
		return 0;
	*priv = NULL;
	return 1;
}

struct matcher firestorm_matchers[]={
	{
		.name = "dns_recursive",
		.desc = "Recursive DNS query",
		.validate = dnsr_validate,
		.new_node = dnsr_new,
		.insert_node = dnsr_insert,
		.cost = MCOST_APP + 2,
	},
	{
		.name = "dns_iterative",
		.desc = "Iterative DNS query",
		.validate = dnsi_validate,
		.new_node = dnsr_new,
		.insert_node = dnsi_insert,
		.cost = MCOST_APP + 2,
		.compatible = "dns_recursive"
	},
	matcher_null()
};

struct plugin_desc firestorm_plugin_desc = {
	.size		= sizeof(firestorm_plugin_desc),
	.name		= "matcher.dns",
	.desc		= "DNS matching routines",
	.author_name	= "Gianni Tedesco",
	.author_email	= "gianni@scaramanga.co.uk",
	.ver_major	= 0,
	.ver_minor	= 1,
};
