/*
 * This file is part of Firestorm NIDS
 * Copyright (c) 2002,2003,2004 Gianni Tedesco
 * This program is released under the terms of the GNU GPL version 2
 *
 * Functions:
 *  o capdev_add() - register a capdev plugin
 *  o capdev_find() - find a capdev plugin by name.
*/
#include <firestorm.h>
#include <loader.h>
#include <args.h>
#include <alert.h>
#include <signature.h>
#include <decode.h>
#include <capdev.h>

struct capdev *capdevs = NULL;

static void capdev_rw(struct nbio *n)
{
}

static void capdev_del(struct nbio *n)
{
	struct source *s = n->priv;
	list_move(&s->list, &s->owner->d_sources);
}

static struct nbio_ops capdev_ops = {
	.read = capdev_rw,
	.write = capdev_rw,
	.dtor = capdev_del,
};

void capdev_fd_add(struct source *s, struct nbio *n)
{
	n->ops = &capdev_ops;
	n->priv = s;
	nbio_add(&s->owner->iothread, n);
	capdev_rw(n);
}

void capdev_fd_inactive(struct source *s, struct nbio *n)
{
	nbio_inactive(&s->owner->iothread, n);
}

void capdev_init_packet(struct packet *p, struct source *s)
{
	p->source = s;
}

/* plugins call this to register capdevs */
static void capdev_add(struct plugin *p, struct capdev *cap)
{
	for(; cap->name; cap++) {
		if ( capdev_find(cap->name) ) {
			mesg(M_DEBUG,"capdev: error: '%s' capdev is "
				"already registered", cap->name);
			continue;
		}
		cap->next=capdevs;
		capdevs=cap;
	}
}

void capdev_load(void)
{
	loader_find_syms("firestorm_capdevs", (loader_cb_t)capdev_add);
}

/* Find a capdev with the specified name */
struct __private capdev *capdev_find(const char *name)
{
	struct capdev *c;

	if ( !name )
		return NULL;

	for(c=capdevs; c; c=c->next)
		if ( !strcmp(c->name, name) )
			return c;

	return NULL;
}
